<?php

declare(strict_types=1);

namespace Drupal\dp_clock_mock\Clock;

use DateTimeImmutable as DTI;
use Drupal\Core\State\StateInterface;
use Psr\Clock\ClockInterface;

/**
 * {@selfdoc}
 */
final readonly class StateClock implements ClockInterface, ClockHandlerInterface {

  private const string STORAGE_KEY = 'date_point.clock.state';

  /**
   * {@selfdoc}
   */
  public function __construct(private StateInterface $state) {

  }

  /**
   * {@inheritdoc}
   */
  public function now(): DTI {
    return new DTI($this->get());
  }

  /**
   * {@inheritdoc}
   */
  public function get(): string {
    return $this->state->get(self::STORAGE_KEY, 'now');
  }

  /**
   * {@selfdoc}
   */
  public function set(string $timestamp): self {
    $this->state->set(self::STORAGE_KEY, $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function reset(): self {
    $this->set('now');
    return $this;
  }

}
