<?php

declare(strict_types=1);

namespace Drupal\Tests\dp_clock_mock\Kernel;

use DateTimeImmutable as DTI;
use Drupal\KernelTests\KernelTestBase;
use Psr\Clock\ClockInterface;

/**
 * A test for a frozen clock.
 *
 * @group date_point
 */
final class FrozenClockTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['dp_clock_mock'];

  /**
   * @dataProvider serviceProvider()
   */
  public function testClock(string $service, string $date_time): void {
    $clock = $this->container->get('dp_clock_mock.clock.' . $service);
    self::assertInstanceOf(ClockInterface::class, $clock);
    self::assertEquals(new DTI($date_time), $clock->now());
  }

  /**
   * {@selfdoc}
   */
  public static function serviceProvider(): \Generator {
    yield ['unix_epoch', '1970-01-01 00:00:00.000000+00:00'];
    yield ['millennium', '2001-01-01 00:00:00.000000+00:00'];
    yield ['tee_time', '17:00:00.000000+00:00'];
    yield ['midnight', '00:00:00.000000+00:00'];
  }

}
