<?php

declare(strict_types=1);

namespace Drupal\date_point\Clock;

use DateTimeImmutable as DTI;
use Drupal\Core\State\StateInterface;
use Psr\Clock\ClockInterface;

/**
 * A clock that allows the datetime to be frozen.
 *
 * @deprecated use \Drupal\dp_clock_mock\Clock\StateClock instead.
 */
final readonly class StateClock implements ClockInterface {

  private const string STORAGE_KEY = 'date_point_state_clock.now';
  private const string STORAGE_FORMAT = 'Y-m-d H:i:s.u';
  private const string STORAGE_TIMEZONE = 'UTC';

  /**
   * {@selfdoc}
   */
  public function __construct(private StateInterface $state) {
    // @phpcs:ignore
    @trigger_error(__CLASS__ . ' is deprecated.', E_USER_DEPRECATED);
  }

  /**
   * {@inheritdoc}
   */
  public function now(): DTI {
    return $this->get();
  }

  /**
   * {@selfdoc}
   */
  public function get(): DTI {
    return new DTI(
      $this->state->get(self::STORAGE_KEY, 'now'), new \DateTimeZone(self::STORAGE_TIMEZONE),
    );
  }

  /**
   * {@selfdoc}
   */
  public function set(DTI $datetime): void {
    $this->save($datetime->setTimezone(new \DateTimeZone(self::STORAGE_TIMEZONE)));
  }

  /**
   * {@selfdoc}
   */
  public function modify(string $modifier): DTI {
    $datetime = $this->now()->modify($modifier);
    $this->save($datetime);
    return $this->get();
  }

  /**
   * {@selfdoc}
   */
  public function reset(): void {
    $this->state->delete(self::STORAGE_KEY);
  }

  /**
   * {@selfdoc}
   */
  private function save(DTI $datetime): void {
    $this->state->set(self::STORAGE_KEY, $datetime->format(self::STORAGE_FORMAT));
  }

}
